# -*- coding: utf-8 -*-
import hashlib
import json
from excalibur.core import Query
from excalibur.exceptions import ExcaliburError
import logging
from .exceptions import excalibur_exception_handler
from rest_framework.authtoken.models import Token
from django.core.exceptions import ObjectDoesNotExist
from britney.errors import SporeMethodStatusError, SporeMethodCallError
from britney.middleware import auth
import britney_utils
from rest_framework.reverse import reverse
from django.core.urlresolvers import NoReverseMatch


"""
Tools for the excalibur middleware

"""

logger = logging.getLogger(__name__)


class ExcaliburAttack():

    """
    params for excalibur
    """

    def __init__(self, request, response):
        # base attribute
        self.request = request
        self.response = response
        # secondary attribute (use request and response)
        self.arguments = self.__arguments()
        self.ressource = self.__ressource()
        self.token = self.__token()
        self.remote_ip = self.__remote_ip()
        self.request_method = self.__request_method()
        self.project = self.__project()
        self.source = self.__source()
        self.data = self.__data()
        self.method = self.__method()
        # signature needs all other params to be set
        self.signature = self.__signature()

    def __arguments(self):
        """ get arguments id from the path """
        try:
            arguments = {}
            arguments["id"] = self. __find_id()
            arguments["project"] = self.__project()
            arguments["establishment"] = self.__source()
            arguments["base_url"] = self.__find_base_url()
            arguments.update(self.__optionnal_args())
        except IndexError:
            return None

        return arguments

    def __find_base_url(self):
        return '%s://%s' % \
                ('https' if self.request.is_secure() else 'http',
                 self.request.get_host())

    def __find_id(self):
        return self.request.META["REQUEST_EXCALIBUR_PARAMS"]["id"]

    def __ressource(self):
        """ get the ressource from the path """
        try:
            return self.request.META["REQUEST_EXCALIBUR_PARAMS"]["ressource"]
        except KeyError:
            return None

    def __method(self):
        """ get the method from the path """
        try:
            return self.request.META["REQUEST_EXCALIBUR_PARAMS"]["method"]
        except KeyError:
            return None

    def __token(self):
        """ get the token from the request header """
        token = self.request.META["HTTP_AUTHORIZATION"].split(' ')[-1] \
            if "HTTP_AUTHORIZATION" in self.request.META.keys() else None
        return token

    def __remote_ip(self):
        """ get remote ip from the request """
        remote_ip = self.request.META[
            'HTTP_X_REAL_IP'] if 'HTTP_X_REAL_IP' in self.request.META \
            else self.request.META['REMOTE_ADDR']
        return remote_ip

    def __request_method(self):
        """ get the request method """
        return self.request.method

    def __project(self):
        """ get the project from the request args """
        return self.request.REQUEST['project']

    def __source(self):
        """ get the source from the request args """
        return self.request.REQUEST['establishment']

    def __optionnal_args(self):
        "get all other args"
        return {k:v for k,v in self.request.REQUEST.items()
                if k not in ['project','establishment']}

    def __data(self):
        """ load data from responce content """
        return json.loads(str(self.response.content, 'utf-8'))

    def __signature(self):
        """ build the excalibur signature with the user's token """
        signkey = None
        if self.token:
            arguments_list = sorted(self.arguments)
            to_hash = self.token
            for argument in arguments_list:
                to_hash += (argument + self.arguments[argument])
            signkey = hashlib.sha1(to_hash.encode("utf-8")).hexdigest()

        return signkey

    def make_and_run_query(self, plugin_runner, etab=None):
        """
        make and run the query depending on etab
        """
        # create the query
        query = Query(
            source=etab if etab else self.source,
            remote_ip=self.remote_ip,
            arguments=self.arguments,
            ressource=self.ressource,
            method=self.method,
            request_method=self.request_method,
            project=self.project,
            signature=self.signature
        )

        newdata, errors = plugin_runner(query)

        return newdata, errors

    def manage_errors(self, errors):
        """ manage errors """
        # log each plugin's errors
        for plugin_name, err_par in errors.items():
            myerror = "error: %s:%s, plugin name: %s, source: %s, \
        ressource: %s, method: %s, arguments: %s, parameters index: %s" % (
                err_par['error'],
                err_par['error_message'],
                plugin_name,
                err_par['source'],
                err_par['ressource'],
                err_par['method'],
                err_par['arguments'],
                err_par['parameters_index'])

            logger.warning(myerror)

        return excalibur_exception_handler(ExcaliburError(myerror),
                                           self.response)

    def aggregate_data(self, newdata):
        """
        Aggregate plugins data to camelot data.
        Priority for the referentiel data, except if the data is null,empty...
        """
        if newdata:

            # aggregate in a dictionnary
            if isinstance(self.data, dict):

                # Remove not found message of referentiel if 404
                if self.response.status_code == 404:
                    self.response.status_code = 200
                    if 'error' in self.data:
                        del self.data['error']

                # Aggregate data if not exists
                for key, values in newdata.items():
                    # update the new dict with data from referentiel
                    if isinstance(values, dict):
                        self.data.update(
                            {k: v for k, v in values.items(
                            ) if k not in self.data.keys()}
                        )
                    # if its a list or other, just add a new key named
                    # like the plugin
                    else:
                        self.data.update({key: values})

            # aggregate in a list
            elif isinstance(self.data, list):
                for values in newdata.values():
                    # if the plugin return a list, merge the two lists
                    if isinstance(values, list):
                        self.data += values
                    # if the plugin return something else, just append
                    else:
                        self.data.append(values)

        self.response.content = json.dumps(self.data)

        return self.response

    def get_request_user(self):
        """
        get the user from the request
        """
        user = None
        token = self.request.META["HTTP_AUTHORIZATION"].split(' ')[-1] \
            if "HTTP_AUTHORIZATION" in self.request.META.keys() else None

        if token:
            try:
                token = Token.objects.get(key=token)
                if token.user:
                    user = token.user
            except ObjectDoesNotExist:
                user = None

        return user


def generate_client_and_get_data(spore, token, method_name, method_args):
    """
    Generate a britney client and get data from method
    """

    client_name = hashlib.sha1(('%s%s' % (spore, token)).encode('utf-8')).hexdigest()
    data = None

    try:
        middlewares = (
            (auth.ApiKey, {'key_name': 'Authorization',
             'key_value': 'Token %s' % (token,)}),
        )

        client = britney_utils.get_client(client_name,
                                          spore,
                                          middlewares=middlewares)

        func = getattr(client, method_name)
        user = func(**method_args)
        data = json.loads(user.text)

    except SporeMethodStatusError:
        raise
    except SporeMethodCallError:
        raise
    else:
        return data


def build_route_updates_data(users_list, project, establishment, member_label, member_code_key, base_url, reverse_name):
    """
    build the updates route
    """

    res_list = []

    try:

        for result in users_list:

            url_reverse = reverse("%s" % reverse_name,
                                  kwargs={'memberskey': member_label,
                                          'memberscode': result[member_code_key]},
                                  format='json', request=None)

            url_args = "?establishment=%s&project=%s" % (establishment,
                                                         project)
            url = "%s%s%s" % (base_url, url_reverse, url_args)

            res_list.append({'project': project,
                             'establishment': establishment,
                             'source': member_label, 'code': result[member_code_key],
                             'url': url})
    except SporeMethodStatusError:
        raise
    except SporeMethodCallError:
        raise
    except KeyError as e:
        logger.critical(str(e))
    except NoReverseMatch as e:
        logger.critical(str(e))

    return res_list



# -*- coding: utf-8 -*-
from functools import wraps

"""
decorators for excalibur
"""


def is_excalibur(func):
    """
    check if excalibur can be called
    """

    def check_content_type(response):
        """ check if the content type of the response is json """
        return response['Content-Type'] == 'application/json'

    def check_request(request):
        """ check the request method and args """
        return request.method == "GET" and 'project' in \
            request.REQUEST.keys() and 'establishment' in request.REQUEST.keys() \
            and "REQUEST_EXCALIBUR_PARAMS" in request.META

    def check_permission(request, response):
        """ check if not 401 or 403 """
        return response.status_code not in (401, 403)

    def check_status_code(response):
        """
        Run excalibur if the response from the referentiel is 200 (data ok) or
        404 (not found, but excalibur try to find it)
        """
        return response.status_code in (200, 404)

    def check_excalibur(request, response):
        """
        return true if the request needs excalibur
        """
        return check_content_type(response) and check_request(request)\
            and check_permission(request, response) and check_status_code(response)

    @wraps(func)
    def wrapper(middleware, request, response):
        if not check_excalibur(request, response):
            return response
        return func(middleware, request, response)

    return wrapper



from django.db import models
import yaml
from django.core.exceptions import ValidationError
from django.conf import settings


class Configuration(models.Model):
    """
    yaml configuration in database
    """
    name = models.CharField(max_length=53, unique=True)
    configuration = models.TextField(blank=True, null=True)

    def save(self, *args, **kwargs):
        try:
            yaml.load(self.configuration)
        except Exception as e:
            raise ValidationError(e)
        else:
            super(Configuration, self).save(*args, **kwargs)
            # reinitialize the excalbur conf singleton
            ExcaliburConf.removeInstance()

    class Meta:
        db_table = 'excalibur_configuration'


class ExcaliburConf():

    """
    load the configuration from database
    """

    instance = None

    class __ExcaliburConf:

        def __init__(self):
            self.sources_conf = Configuration.objects.get(
                name=settings.EXCALIBUR_SOURCES)
            self.ressource_conf = Configuration.objects.get(
                name=settings.EXCALIBUR_RESSOURCES)
            self.acl_conf = Configuration.objects.get(
                name=settings.EXCALIBUR_ACL)
            self.plugins_module = settings.EXCALIBUR_PLUGINS_MODULE

    def __new__(cls):
        if not ExcaliburConf.instance:
            ExcaliburConf.instance = ExcaliburConf.__ExcaliburConf()
        return ExcaliburConf.instance

    def __getattr__(self, name):
        return getattr(self.instance, name)

    def __setattr__(self, name):
        return setattr(self.instance, name)

    @staticmethod
    def removeInstance():
        ExcaliburConf.instance = None


# -*- coding: utf-8 -*-
from django.test import TestCase
from .decorators import is_excalibur
from .middleware import ExcaliburMiddleware
from .exceptions import excalibur_exception_handler
from django.http import HttpRequest
from django.http import HttpResponse
from django.http import QueryDict
from rest_framework.request import Request
from rest_framework.response import Response
from django.contrib.auth.models import User
from rest_framework.authtoken.models import Token
from .models import Configuration
from django.core.exceptions import ValidationError
from .utils import ExcaliburAttack
from .models import ExcaliburConf
from excalibur.core import PluginsRunner
from mock import patch, Mock
import json
from django.conf import settings


ACL = """
proj:
    myetab:
        members:
            - method1
            - method2
            - method3
    myetab2:
        members:
            - method2
"""

RESSOURCES = """
members:
    method1:
        request method: GET
        arguments:
            id:
                checks:
                    min length: 1
                    max length: 50
            project:
                checks:
                    min length: 1
                    max length: 50
            establishment:
                checks:
                    min length: 1
                    max length: 50
            base_url:
                checks:
                    min length: 1
                    max length: 50

    method2:
        request method: GET
        arguments:
            id:
                checks:
                    min length: 1
                    max length: 50
            project:
                checks:
                    min length: 1
                    max length: 50
            establishment:
                checks:
                    min length: 1
                    max length: 50
            base_url:
                checks:
                    min length: 1
                    max length: 50

    method3:
        request method: GET
        arguments:
            id:
                checks:
                    min length: 1
                    max length: 50
            project:
                checks:
                    min length: 1
                    max length: 50
            establishment:
                checks:
                    min length: 1
                    max length: 50
            base_url:
                checks:
                    min length: 1
                    max length: 50
"""

SOURCES = """
proj:
     sources:
        myetab:
            plugins:

                Ldapuds:
                    -   spore: http://myurl/description.json
                        token: S3CR3T

                Apogee:
                    -   spore: http://myurl/description.json
                        token: S3CR3T

                Harpege:
                    -   spore: http://myurl/description.json
                        token: S3CR3T

        myetab2:
            plugins:

                Bnu:
                    -   spore: http://myurl/description.json
                        token: S3CR3T
"""


class DecoratorIsExcaliburTest(TestCase):

    """
    Decorator isExcalibur test
    """

    def setUp(self):
        # create a mock decorator
        self.function = Mock()
        self.function.__name__ = 'mock'
        self.decorated_function = is_excalibur(self.function)
        self.middleware = ExcaliburMiddleware()
        # create user with token
        self.user = User.objects.create(username="arthur")
        self.token = Token.objects.create(
            user=self.user,
            key=b'S3CR3T')
        # create a right request
        self.httprequest = HttpRequest()
        self.httprequest.method = "GET"
        self.httprequest.encoding = 'utf-8'
        self.httprequest.GET = QueryDict("project=proj&establishment=myetab")
        self.httprequest.REQUEST = self.httprequest.GET
        self.httprequest.META["HTTP_AUTHORIZATION"] = "Token " + \
            self.token.key.decode(encoding='UTF-8')
        self.httprequest.META['REQUEST_EXCALIBUR_PARAMS'] = {
            "ressource": "members",
            "method": "method2",
            "id": "32"
        }
        self.request = Request(self.httprequest)
        # create a right response
        self.httpresponse = HttpResponse()
        self.response = Response(self.httpresponse)
        self.response['Content-Type'] = 'application/json'
        self.response['encoding'] = 'utf-8'
        self.response.status_code = 200

    def test_ok(self):
        self.decorated_function(self.middleware, self.request, self.response)
        self.assertTrue(self.function.called)

    def test_wrong_status_code(self):
        self.response.status_code = 500
        self.decorated_function(self.middleware, self.request, self.response)
        self.assertFalse(self.function.called)

    def test_wrong_content_type(self):
        self.response['Content-Type'] = 'text/html'
        self.decorated_function(self.middleware, self.request, self.response)
        self.assertFalse(self.function.called)

    def test_wrong_method(self):
        self.httprequest.method = "POST"
        self.request = Request(self.httprequest)
        self.decorated_function(self.middleware, self.request, self.response)
        self.assertFalse(self.function.called)

    def test_wrong_args(self):
        self.httprequest.GET = QueryDict("error=error")
        self.httprequest.REQUEST = self.httprequest.GET
        self.request = Request(self.httprequest)
        self.decorated_function(self.middleware, self.request, self.response)
        self.assertFalse(self.function.called)

    def tearDown(self):
        self.function = None
        self.decorated_function = None
        self.request = None
        self.response = None


class ExceptionTest(TestCase):
    """
    test the custom exception of excalibur
    """

    def setUp(self):
        self.exception = Exception("FATAL ERROR")
        # create a right response
        self.httpresponse = HttpResponse()
        self.response = Response(self.httpresponse)
        self.response['Content-Type'] = 'application/json'
        self.response['encoding'] = 'utf-8'
        self.response.status_code = 200

    def test_exception_handler(self):
        response_err = excalibur_exception_handler(self.exception,
                                                   self.response)
        self.assertIsInstance(response_err, Response)
        self.assertEqual(response_err.status_code, 500)
        self.assertEqual(response_err.content.decode('utf-8'),
                         '{"error": "FATAL ERROR"}')


class ConfigurationModelTest(TestCase):
    """
    test for the configuration Model
    """

    def setUp(self):
        self.conf_acl = Configuration.objects.create(name="acl.yml",
                                                     configuration=ACL)
        self.conf_ressources = Configuration.objects.create(
            name="ressources.yml",
            configuration=RESSOURCES)
        self.conf_sources = Configuration.objects.create(
            name="sources.yml",
            configuration=SOURCES)

    def test_get(self):
        self.assertEqual(self.conf_acl,
                         Configuration.objects.get(name="acl.yml"))
        self.assertEqual(self.conf_ressources,
                         Configuration.objects.get(name="ressources.yml"))
        self.assertEqual(self.conf_sources,
                         Configuration.objects.get(name="sources.yml"))

    def test_yaml_error(self):
        with self.assertRaises(ValidationError):
            Configuration.objects.create(name="error.notayml",
                                         configuration="iamnoymal\t")

    def test_excalibur_conf(self):
        ExcaliburConf()
        self.assertIsNotNone(ExcaliburConf.instance)

    def tearDown(self):
        ExcaliburConf.removeInstance()


class ExcaliburAttackAndMiddlewareTest(TestCase):
    """
    test the excalbur attack class
    """

    plugin_mock1 = Mock()
    instance = plugin_mock1.return_value
    instance.members_method2.return_value = {"key1": "val1"}

    plugin_mock2 = Mock()
    instance = plugin_mock2.return_value
    instance.members_method2.return_value = {"key2": "val2"}

    plugin_mock3 = Mock()
    instance = plugin_mock3.return_value
    instance.members_method2.return_value = {"key3": "val3"}

    plugin_mock4 = Mock()
    instance = plugin_mock4.return_value
    instance.members_method2.return_value = ["item1", "item2"]

    plugin_mock5 = Mock()
    instance = plugin_mock5.return_value
    instance.members_method2.return_value = ["item3", "item4"]

    plugin_mock6 = Mock()
    instance = plugin_mock6.return_value
    instance.members_method2.return_value = ["item5", "item6"]

    plugin_mock7 = Mock()
    instance = plugin_mock7.return_value
    instance.members_method2.return_value = {"key11": "val11"}

    def raiseException(*args, **kwargs):
        raise Exception()
    plugin_mock8 = Mock()
    instance = plugin_mock8.return_value
    instance.members_method2 = raiseException

    def setUp(self):
        # create user with token
        self.user = User.objects.create(username="arthur")
        self.token = Token.objects.create(
            user=self.user,
            key=b'S3CR3T')
        # create a right request
        self.httprequest = HttpRequest()
        self.httprequest.path = "/warehouse/members/method2/32.json?\
project=sigb&establishment=uds"
        self.httprequest.method = "GET"
        self.httprequest.encoding = 'utf-8'
        self.httprequest.GET = QueryDict("project=proj&establishment=myetab")
        self.httprequest.REQUEST = self.httprequest.GET
        self.httprequest.META["REMOTE_ADDR"] = "127.0.0.1"
        self.httprequest.META["HTTP_AUTHORIZATION"] = "Token " + \
            self.token.key.decode(encoding='UTF-8')
        self.httprequest.META['SERVER_NAME'] = "myserverurl.com"
        self.httprequest.META['SERVER_PORT'] = 80
        self.httprequest.META['REQUEST_EXCALIBUR_PARAMS'] = {
            "ressource": "members",
            "method": "method2",
            "id": "32"
        }
        self.request = Request(self.httprequest)
        # create a right response
        self.httpresponse = HttpResponse()
        self.response = Response(self.httpresponse)
        self.response['Content-Type'] = 'application/json'
        self.response['encoding'] = 'utf-8'
        self.response.status_code = 200
        self.response.content = '{"key1":"prioritary","name":"toto","age":18}'
        # conf
        self.conf_acl = Configuration.objects.create(name="acl.yml",
                                                     configuration=ACL)
        self.conf_ressources = Configuration.objects.create(
            name="ressources.yml",
            configuration=RESSOURCES)
        self.conf_sources = Configuration.objects.create(
            name="sources.yml",
            configuration=SOURCES)
        self.attack = ExcaliburAttack(self.request, self.response)
        self.middleware = ExcaliburMiddleware()
        # Excalibur Conf
        self.excconf = ExcaliburConf()

    def test_attack_ok(self):
        self.assertEqual(self.attack.request, self.request)
        self.assertEqual(self.attack.response, self.response)
        self.assertEqual(
            self.attack.arguments,
            {'project': 'proj', 'establishment': 'myetab', 'id': '32',
             'base_url': 'http://myserverurl.com'})
        self.assertEqual(self.attack.ressource, "members")
        self.assertEqual(self.attack.token, self.token.key.decode('utf-8'))
        self.assertEqual(self.attack.remote_ip, "127.0.0.1")
        self.assertEqual(self.attack.request_method, "GET")
        self.assertEqual(self.attack.project, "proj")
        self.assertEqual(self.attack.source, "myetab")
        self.assertEqual(self.attack.data, {'key1': 'prioritary',
                                            'age': 18, 'name': 'toto'})
        self.assertEqual(self.attack.method, "method2")
        self.assertEqual(self.attack.signature,
                         "d7b3487b278d076467a1482eeb6432980a0e49e8")

    @patch('%s.Harpege.Harpege' % settings.EXCALIBUR_PLUGINS_MODULE, plugin_mock1)
    @patch('%s.Apogee.Apogee' % settings.EXCALIBUR_PLUGINS_MODULE, plugin_mock2)
    @patch('%s.Ldapuds.Ldapuds' % settings.EXCALIBUR_PLUGINS_MODULE, plugin_mock3)
    def test_make_and_run_query(self):

        plugin_runner = PluginsRunner(
            self.excconf.acl_conf.configuration,
            self.excconf.sources_conf.configuration,
            self.excconf.ressource_conf.configuration,
            self.excconf.plugins_module,
            raw_yaml_content=True
        )

        newdata, errors = self.attack.make_and_run_query(plugin_runner)

        self.assertEqual(errors, {})
        self.assertEqual(
            newdata,
            {'Harpege': {'key1': 'val1'}, 'Ldapuds': {'key3': 'val3'},
             'Apogee': {'key2': 'val2'}})

    @patch('%s.Harpege.Harpege' % settings.EXCALIBUR_PLUGINS_MODULE, plugin_mock8)
    @patch('%s.Apogee.Apogee' % settings.EXCALIBUR_PLUGINS_MODULE, plugin_mock8)
    @patch('%s.Ldapuds.Ldapuds' % settings.EXCALIBUR_PLUGINS_MODULE, plugin_mock8)
    def test_manage_errors(self):

        plugin_runner = PluginsRunner(
            self.excconf.acl_conf.configuration,
            self.excconf.sources_conf.configuration,
            self.excconf.ressource_conf.configuration,
            self.excconf.plugins_module,
            raw_yaml_content=True
        )

        newdata, errors = self.attack.make_and_run_query(plugin_runner)

        self.assertEqual(newdata, {})
        self.assertTrue(errors)

    @patch('%s.Harpege.Harpege' % settings.EXCALIBUR_PLUGINS_MODULE, plugin_mock1)
    @patch('%s.Apogee.Apogee' % settings.EXCALIBUR_PLUGINS_MODULE, plugin_mock2)
    @patch('%s.Ldapuds.Ldapuds' % settings.EXCALIBUR_PLUGINS_MODULE, plugin_mock3)
    def test_aggregate_data_dict_dict(self):

        plugin_runner = PluginsRunner(
            self.excconf.acl_conf.configuration,
            self.excconf.sources_conf.configuration,
            self.excconf.ressource_conf.configuration,
            self.excconf.plugins_module,
            raw_yaml_content=True
        )

        newdata, errors = self.attack.make_and_run_query(plugin_runner)

        response = self.attack.aggregate_data(newdata)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            json.loads(response.content.decode('utf-8')),
            {"key3": "val3", "key2": "val2", "age": 18,
             "name": "toto", "key1": "prioritary"})

    @patch('%s.Harpege.Harpege' % settings.EXCALIBUR_PLUGINS_MODULE, plugin_mock1)
    @patch('%s.Apogee.Apogee' % settings.EXCALIBUR_PLUGINS_MODULE, plugin_mock2)
    @patch('%s.Ldapuds.Ldapuds' % settings.EXCALIBUR_PLUGINS_MODULE, plugin_mock3)
    def test_aggregate_data_with_404(self):
        self.response.status_code = 404
        self.response.content = '{"error":"404 not found"}'
        self.attack = ExcaliburAttack(self.request, self.response)

        plugin_runner = PluginsRunner(
            self.excconf.acl_conf.configuration,
            self.excconf.sources_conf.configuration,
            self.excconf.ressource_conf.configuration,
            self.excconf.plugins_module,
            raw_yaml_content=True
        )

        newdata, errors = self.attack.make_and_run_query(plugin_runner)

        response = self.attack.aggregate_data(newdata)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            json.loads(response.content.decode('utf-8')),
            {"key3": "val3", "key2": "val2", "key1": "val1"})

    @patch('%s.Harpege.Harpege' % settings.EXCALIBUR_PLUGINS_MODULE, plugin_mock1)
    @patch('%s.Apogee.Apogee' % settings.EXCALIBUR_PLUGINS_MODULE, plugin_mock2)
    @patch('%s.Ldapuds.Ldapuds' % settings.EXCALIBUR_PLUGINS_MODULE, plugin_mock3)
    def test_aggregate_data_with_list_dict(self):
        self.response.content = '["item1", "item2"]'
        self.attack = ExcaliburAttack(self.request, self.response)


        plugin_runner = PluginsRunner(
            self.excconf.acl_conf.configuration,
            self.excconf.sources_conf.configuration,
            self.excconf.ressource_conf.configuration,
            self.excconf.plugins_module,
            raw_yaml_content=True
        )

        newdata, errors = self.attack.make_and_run_query(plugin_runner)

        response = self.attack.aggregate_data(newdata)
        self.assertEqual(response.status_code, 200)
#         self.assertEqual(
#             json.loads(response.content.decode('utf-8')),
#             ['item1', 'item2', {'key1': 'val1'}, {'key3': 'val3'},
#                                {'key2': 'val2'}])
        items = ['item1', 'item2', {'key1': 'val1'}, {'key3': 'val3'},
                               {'key2': 'val2'}]
        loaded_json = json.loads(response.content.decode('utf-8'))
        not_found = lambda x : x[0] not in x[1]
        not_found_list = [thing for thing in items \
                          if not_found([thing,loaded_json])]
        self.assertEqual([],not_found_list) 

    @patch('%s.Harpege.Harpege' % settings.EXCALIBUR_PLUGINS_MODULE, plugin_mock4)
    @patch('%s.Apogee.Apogee' % settings.EXCALIBUR_PLUGINS_MODULE, plugin_mock5)
    @patch('%s.Ldapuds.Ldapuds' % settings.EXCALIBUR_PLUGINS_MODULE, plugin_mock6)
    def test_aggregate_data_with_list_list(self):
        self.response.content = '["item99", "item98"]'
        self.attack = ExcaliburAttack(self.request, self.response)


        plugin_runner = PluginsRunner(
            self.excconf.acl_conf.configuration,
            self.excconf.sources_conf.configuration,
            self.excconf.ressource_conf.configuration,
            self.excconf.plugins_module,
            raw_yaml_content=True
        )

        newdata, errors = self.attack.make_and_run_query(plugin_runner)

        response = self.attack.aggregate_data(newdata)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            json.loads(response.content.decode('utf-8')).sort(),
            ["item99", "item98", "item1", "item2", "item5", "item6",
             "item3", "item4"].sort())

    @patch('%s.Harpege.Harpege' % settings.EXCALIBUR_PLUGINS_MODULE, plugin_mock4)
    @patch('%s.Apogee.Apogee' % settings.EXCALIBUR_PLUGINS_MODULE, plugin_mock5)
    @patch('%s.Ldapuds.Ldapuds' % settings.EXCALIBUR_PLUGINS_MODULE, plugin_mock6)
    def test_aggregate_data_with_dict_list(self):
        self.attack = ExcaliburAttack(self.request, self.response)

        plugin_runner = PluginsRunner(
            self.excconf.acl_conf.configuration,
            self.excconf.sources_conf.configuration,
            self.excconf.ressource_conf.configuration,
            self.excconf.plugins_module,
            raw_yaml_content=True
        )

        newdata, errors = self.attack.make_and_run_query(plugin_runner)

        response = self.attack.aggregate_data(newdata)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            json.loads(response.content.decode('utf-8')),
            {'name': 'toto', 'age': 18, 'Apogee': ['item3', 'item4'],
             'Harpege': ['item1', 'item2'], 'Ldapuds': ['item5', 'item6'],
             'key1': 'prioritary'})

    @patch('%s.Harpege.Harpege' % settings.EXCALIBUR_PLUGINS_MODULE, plugin_mock1)
    @patch('%s.Apogee.Apogee' % settings.EXCALIBUR_PLUGINS_MODULE, plugin_mock2)
    @patch('%s.Ldapuds.Ldapuds' % settings.EXCALIBUR_PLUGINS_MODULE, plugin_mock3)
    def test_middleware_no_is_excalbur(self):
        self.response['Content-Type'] = 'text/html'
        response = self.middleware.process_response(self.request,
                                                    self.response)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            json.loads(response.content.decode('utf-8')),
            {"key1": "prioritary", "name": "toto", "age": 18})

    @patch('%s.Harpege.Harpege' % settings.EXCALIBUR_PLUGINS_MODULE, plugin_mock8)
    @patch('%s.Apogee.Apogee' % settings.EXCALIBUR_PLUGINS_MODULE, plugin_mock8)
    @patch('%s.Ldapuds.Ldapuds' % settings.EXCALIBUR_PLUGINS_MODULE, plugin_mock8)
    def test_middleware_with_errors(self):
        response = self.middleware.process_response(self.request,
                                                    self.response)
        self.assertEqual(response.status_code, 500)
        self.assertTrue('error' in json.loads(
            response.content.decode('utf-8')).keys())

    @patch('%s.Harpege.Harpege' % settings.EXCALIBUR_PLUGINS_MODULE, plugin_mock1)
    @patch('%s.Apogee.Apogee' % settings.EXCALIBUR_PLUGINS_MODULE, plugin_mock2)
    @patch('%s.Ldapuds.Ldapuds' % settings.EXCALIBUR_PLUGINS_MODULE, plugin_mock3)
    def test_middleware_one_establishement(self):
        response = self.middleware.process_response(self.request,
                                                    self.response)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            json.loads(response.content.decode('utf-8')),
            {"key3": "val3", "age": 18, "key1": "prioritary",
             "name": "toto", "key2": "val2"})

    @patch('%s.Harpege.Harpege' % settings.EXCALIBUR_PLUGINS_MODULE, plugin_mock1)
    @patch('%s.Apogee.Apogee' % settings.EXCALIBUR_PLUGINS_MODULE, plugin_mock2)
    @patch('%s.Ldapuds.Ldapuds' % settings.EXCALIBUR_PLUGINS_MODULE, plugin_mock3)
    @patch('%s.Bnu.Bnu' % settings.EXCALIBUR_PLUGINS_MODULE, plugin_mock7)
    def test_middleware_all_establishement(self):
        self.httprequest.GET = QueryDict("project=proj&establishment=all")
        self.httprequest.REQUEST = self.httprequest.GET
        response = self.middleware.process_response(self.request,
                                                    self.response)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            json.loads(response.content.decode('utf-8')),
            {"name": "toto", "key3": "val3", "key2": "val2",
             "age": 18, "key1": "prioritary", "key11": "val11"})

    @patch('%s.Harpege.Harpege' % settings.EXCALIBUR_PLUGINS_MODULE, plugin_mock1)
    @patch('%s.Apogee.Apogee' % settings.EXCALIBUR_PLUGINS_MODULE, plugin_mock2)
    @patch('%s.Ldapuds.Ldapuds' % settings.EXCALIBUR_PLUGINS_MODULE, plugin_mock3)
    def test_middleware_wrong_auth(self):
        self.response.status_code = 404
        self.response.content = "{\"error\": \"not found\"}"
        self.request.META["HTTP_AUTHORIZATION"] = "Token ERROR"
        response = self.middleware.process_response(self.request,
                                                    self.response)
        self.assertEqual(response.status_code, 404)
        self.assertEqual(json.loads(response.content.decode('utf-8')),
                         {"error": "not found"})

    @patch('%s.Harpege.Harpege' % settings.EXCALIBUR_PLUGINS_MODULE, plugin_mock1)
    @patch('%s.Apogee.Apogee' % settings.EXCALIBUR_PLUGINS_MODULE, plugin_mock2)
    @patch('%s.Ldapuds.Ldapuds' % settings.EXCALIBUR_PLUGINS_MODULE, plugin_mock3)
    def test_middleware_no_auth(self):
        self.response.status_code = 404
        self.response.content = "{\"error\": \"not found\"}"
        self.request.META.pop("HTTP_AUTHORIZATION", None)
        response = self.middleware.process_response(self.request,
                                                    self.response)
        self.assertEqual(response.status_code, 404)
        self.assertEqual(json.loads(response.content.decode('utf-8')),
                         {"error": "not found"})

    def tearDown(self):
        ExcaliburConf.removeInstance()



import json

"""
Excalibur exceptions handler
"""


def excalibur_exception_handler(exc, response):
    """
    Return excalibur error response
    """

    response.content = json.dumps({'error': str(exc)})
    response.status_code = 500

    return response


# -*- coding: utf-8 -*-
from excalibur.core import PluginsRunner
from excalibur.exceptions import ConfigurationLoaderError, ExcaliburError
import logging
from .decorators import is_excalibur
from .models import ExcaliburConf
from .utils import ExcaliburAttack
from .exceptions import excalibur_exception_handler
from django.core.exceptions import ObjectDoesNotExist

logger = logging.getLogger(__name__)


class ExcaliburMiddleware(object):

    """ The excalibur middleware """

    @is_excalibur
    def process_response(self, request, response):
        """
            use excalibur when optional "project" and "source" params
            are in the url
        """
        try:
            # get all params for query
            excalibur = ExcaliburAttack(request, response)
            # get user
            user = excalibur.get_request_user()

            if user:
                try:
                    excconf = ExcaliburConf()
                except (AttributeError, ObjectDoesNotExist) as e:
                    raise ConfigurationLoaderError(str(e))

                # run plugins
                plugin_runner = PluginsRunner(
                    excconf.acl_conf.configuration,
                    excconf.sources_conf.configuration,
                    excconf.ressource_conf.configuration,
                    excconf.plugins_module,
                    raw_yaml_content=True,
                    check_signature=False if user.is_superuser else True
                )

                newdata, errors = excalibur.make_and_run_query(plugin_runner)
                if not errors:
                    response = excalibur.aggregate_data(newdata)
                else:
                    response = excalibur.manage_errors(errors)

        except ExcaliburError as e:
            logger.error(e.message)
            response = excalibur_exception_handler(e, response)

        return response



from django.contrib import admin
from .models import Configuration
from django import forms


class ConfigurationAdminForm(forms.ModelForm):

    class Meta:
        model = Configuration
        exclude = []

    def __init__(self, *args, **kwargs):
        super(ConfigurationAdminForm, self).__init__(*args, **kwargs)
        instance = getattr(self, 'instance', None)
        if instance and instance.pk:
            self.fields['name'].widget.attrs['readonly'] = True
            self.fields['configuration'].widget = forms.Textarea(
                attrs={'cols': 80, 'rows': 40})


class ConfigurationAdmin(admin.ModelAdmin):
    form = ConfigurationAdminForm
    list_display = ('name',)
    list_filter = ('name', )
    ordering = ('name',)


admin.site.register(Configuration, ConfigurationAdmin)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup
from setuptools import find_packages


with open('README.rst') as readme:
    long_description = readme.read()


with open('requirements.txt') as requirements:
    lines = requirements.readlines()
    libraries = [lib for lib in lines if not lib.startswith('-')]
    dependency_links = [link.split()[1] for link in lines if
        link.startswith('-f')]


setup(
    name='django-excalibur',
    version='0.0.1',
    packages=find_packages(),
    install_requires=libraries,
    dependency_links=dependency_links,
    long_description=long_description,
    author='Morgan Bohn',
    author_email='morgan.bohn@unistra.fr',
    maintainer='Morgan Bohn',
    maintainer_email='morgan.bohn@unistra.fr',
    description='Django Rest Framework implementation for Excalibur : A tool to manage plugins',
    keywords=['django', 'excalibur', 'plugins'],
    url='https://github.com/unistra/django-excalibur',
    download_url='https://github.com/unistra/django-excalibur',
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
    ]
)
